//  PRIMElaunch-gnome: extension.js
//  Copyright (C) 2013 Tomas Marko Miljenović <TomasM@tomasm.tk>
//
//  This file is part of PRIMElaunch, a collection of GL launchers for various
//  free desktop environments.
//
//  PRIMElaunch is free software: you can redistribute it and/or modify
//  it under the terms of either:
//  (a) the GNU Affero General Public License as published by the
//      Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//  (b) the GNU General Public License as published by the
//      Free Software Foundation, either version 2 of the License, or
//      (at your option) any later version.
//
//  PRIMElaunch is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  and GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>.

//external
const Clutter = imports.gi.Clutter
const GLib = imports.gi.GLib
const Gtk = imports.gi.Gtk
const Lang = imports.lang
const St = imports.gi.St
const Signals = imports.signals
//internal
const Main = imports.ui.main
const MessageTray = imports.ui.messageTray
const PanelMenu = imports.ui.panelMenu
const PopupMenu = imports.ui.popupMenu
const Tweener = imports.ui.tweener
const Util = imports.misc.util


//Once DRI_PRIME use changes, we'll need to do some better parsing
//to populate this

let cardtypes = {
    "igd" : 0,
    "dis" : 1
}
// 
let fileTypes = {
    "desktopFile" : 0
}

let activeCard = cardtypes.igd

let desktopFile, envVars


let tbutton, globalEnvSwitch

//IFDEF $GNOME_VERSION?
//function main() {
//    init()
//}

function launch(uri) {
    //Right now, there are only two options: DRI_PRIME={0,1};
    // this will change.
    print("LAUNCH")
    var argsArray = ["DRI_PRIME="+activeCard.toString()].concat(uri)
    
//     let [random, glxinfoText] = GLib.spawn_command_line_sync(
//           "/usr/bin/env \""+argsArray.join(" ")+"\""
//         ) //is 'random' the exit status?
    
    Util.spawnCommandLine("/bin/sh -c \""+argsArray.join(" ")+"\"")
}

function parseDesktopFile(iojob,fileData) {
    var launchExitCode = false
    console.log("HERE: "+desktopFile)
    //Eventually, parse 'Path=' etc
    var execLine = desktopFile.match(/Exec\=(.+)/)[1].toString()
    console.log(execLine+"\n\n\n")
    launchExitCode = root.launch(execLine.split(/\s+/))
    desktopFile = ""
}

const Card

//Using PopupMenu and PopupBaseMenuItem as a base class is probably not the
// best-in-class approach, but works.
const GPUInfoLayout = new Lang.Class({
    Name: 'PRIMElaunch_gpuInfoLayout',
    Extends: PopupMenu.PopupBaseMenuItem,
    _init: function(params){
        this.parent(params)
        this.layout = new St.BoxLayout()
        this.addActor(this.layout)
    },
    addGPU: function(gpuCardtype){
        let [random, glxinfoText] = GLib.spawn_command_line_sync(
          "sh -c \""+"DRI_PRIME="+gpuCardtype.toString()
          +" glxinfo | grep -A 2 renderer\""
        )
        let gpuCell = new GPUInfoCell(glxinfoText.toString() ,gpuCardtype,this)
        this.layout.add(gpuCell, St.Align.MIDDLE)
        this.connect('clicked', Lang.bind(gpuCell, gpuCell._setInactive))
        gpuCell._onCompleted()
        this.actor.width = (this._children.length > 1) ? this._children.length*gpuCell.pic.width*3.1 : gpuCell.pic.width*4.5
        this.layout.height = ( gpuCell.height > this.layout.height ) ? gpuCell.height : this.layout.height
        
    },
    _onClicked: function(){
      this.emit('clicked')
    }
})
const GPUInfoCell = new Lang.Class ({
    Name: 'PRIMElaunch_gpuInfoCell',
    Extends: St.Button,
    _init: function(glxinfoText,gpuCardtype,parentLayout){
        this.parent({
          style_class: 'panel-button',
          reactive: true,
          track_hover: true
        })
        this.layout = parentLayout
        this.gpuCardtype = gpuCardtype
        
        this.buttonLayout = new St.BoxLayout({
          vertical: true
        })
        this.cardtype = gpuCardtype
        this.renderText = new St.Label({
            text: glxinfoText.replace("\n",": ").split(": ")[1]
        })
        this.renderText.clutter_text.line_wrap = true
        
        //Because in 2005, video-card was removed from the spec
        this.pic = new St.Icon({
            icon_name: 'audio-card'  
        })
        this.buttonLayout.add_actor(this.pic)
        this.buttonLayout.add_actor(this.renderText)
        
        this.set_child(this.buttonLayout)      
        //this.layout.connect('button-press-event', Lang.bind(this, this._setInactive))
        this.connect('button-press-event', Lang.bind(this.layout, this.layout._onClicked))
        this.connect('button-press-event', Lang.bind(this, this._onClicked));
        if(this.gpuCardtype !== activeCard){
            this._setInactive()
        } else {
            this._setActive()
        }
    },
    _onClicked: function(){
      this._setActive()
      activeCard = this.gpuCardtype
      _setPRIME()
    },
    _onCompleted: function(){
      this.renderText.style = (
        'max-width: '+(this.pic.width*3.1).toString()+'px;'
        + 'text-align: center'
      )
    },
    _setActive: function(){
      //in future use get_theme_node() or style_class once documented
      this.style = ('background-color: #4A90D9')
    },
    _setInactive: function(){
        this.style = ('background-color: none')
    }
})

const CmdLayout = new Lang.Class({
    Name: 'PRIMElaunch_cmdLayout',
    Extends: PopupMenu.PopupBaseMenuItem,
    _init: function(params){
        this.parent(params)
        this.layout = new St.BoxLayout()
        
        let promptLabel = new St.Label({
            text: "prompt $ "
        })
        this.cmdField = new St.Entry({
            hint_text: ("Type command..."),
            track_hover: true
        })
        
        this.layout.add(promptLabel)
        this.layout.add(this.cmdField)
        
        this.cmdField.clutter_text.connect('key-press-event', Lang.bind(this, this._onKeyPressed))  
        this.connect('key-press-event', Lang.bind(this, this._onKeyPressed))  
        this.connect('clicked', Lang.bind(this, this._onClicked))
        this.addActor(this.layout)
    },
    _onClicked: function() {
        print("CLICK")
    },
    _onKeyPressed: function(sender, keyPressEvent) { //I think
        switch(keyPressEvent.get_key_symbol()){
            case Clutter.Return :
            case Clutter.KP_Enter :
                this._send()
                this.cmdField.text=""
                break
            default:
                //soon...
                break
        }
    },
    _send: function(){
        launch( this.cmdField.get_text().split(/\s+/) )
    }
})

const DragDropTarget = new Lang.Class({
    Name: 'PRIMElaunch_dragDropTarget',
    Extends: PopupMenu.PopupBaseMenuItem,
    _init: function(params){
        this.parent(params)
        this.layout = new Clutter.Group()//St.BoxLayout()
        this.statText = new St.Label({text: "Drop app icon or uri here"})
        
        //in future use get_theme_node() or style_class once documented
        // to set colours
        this.rectangle = new Clutter.Rectangle({
            x:-6,
            y:-2,
            border_width: 2,
            color: Clutter.Color.get_static(Clutter.StaticColor.BLACK),
            border_color: Clutter.Color.get_static(Clutter.StaticColor.WHITE)
        })
        this.layout.add_actor(this.rectangle, St.Align.MIDDLE)
        this.layout.add_actor(this.statText, St.Align.MIDDLE)
        this.addActor(this.layout)
        this.connect('activate', Lang.bind(this, this._onClicked))
    },
    _onClicked: function(){      
        Main.notify("PRIMElaunch", "Drag and drop: not yet implemented for this DE")
    },
    _onCompleted: function(width){
        this.layout.width = width
        this.layout.height = this.layout.height*2.5
        this.rectangle.width = this.statText.width+12
        this.rectangle.height =  this.statText.height+4
        this.statText.x = (this.layout.width-this.statText.width)/2
        this.statText.y = (this.layout.height-this.statText.height)/2
        this.rectangle.x = this.statText.x - 6
        this.rectangle.y = this.statText.y - 2
    }
})

function _setPRIME() {
    if(globalEnvSwitch.state)
        GLib.setenv("DRI_PRIME",activeCard.toString(),true)
}

function _showAbout() {
    Util.spawnCommandLine("xdg-open http://apps.tomasm.tk/PRIMElaunch/")
}

function _setup() {
    let appFdoIcon = new St.Icon({
        icon_name: 'audio-card',
        style_class: 'system-status-icon'
    })
    let launcherText = new St.Label({
        text: " PRIMElaunch",
    })
    let row = new St.BoxLayout()
    
    row.add_actor(appFdoIcon)
    row.add_actor(launcherText)
    
    tbutton = new PanelMenu.Button(0.6)
    tbutton.actor.add_actor(row)
    
    tbutton.menu.addMenuItem(new CmdLayout({
                                reactive: false,
                                hover: true,
                                can_focus: true,
                                sensitive: false
                            })
    )
    
    let dropArea = new DragDropTarget()
    tbutton.menu.addMenuItem( dropArea )
    
    let gpuInfoLayout = new GPUInfoLayout({reactive: true})
    tbutton.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem())
    tbutton.menu.addMenuItem(gpuInfoLayout)
    /*If you're 'sure' PRIME is supported on your install, but have lspci issues
    (or just don't want to wake your GPU on init() ) uncomment the following
    and nuke the if(..) statements*/
//     gpuInfoLayout.addGPU(cardtypes.igd)
//     gpuInfoLayout.addGPU(cardtypes.dis)
    
        let [random, numGPUs] = GLib.spawn_command_line_sync("/bin/sh -c \"lspci | grep VGA | wc -l\"")
        if(numGPUs >= 1)
            gpuInfoLayout.addGPU(cardtypes.igd)
        if(numGPUs >=2)
            gpuInfoLayout.addGPU(cardtypes.dis)
        if(numGPUs <=0) {
            //FAIL, headless/broken/advanced demon
            disable()
        }
    
    globalEnvSwitch = new PopupMenu.PopupSwitchMenuItem(
        "Set globally",
        true,
        {
            reactive: true,
            hover: true,
            can_focus: true,
            sensitive: false 
        }
    )

    globalEnvSwitch.removeActor(globalEnvSwitch.label)
    globalEnvSwitch.removeActor(globalEnvSwitch._statusBin)
    
    let switchLayout = new St.BoxLayout()
    let switchLabelLayout = new St.BoxLayout({vertical: true})
    switchLabelLayout.add(globalEnvSwitch.label)
    switchLabelLayout.add( new St.Label({
                    text: "WARNING: May irritate Clutter",
                    style: "font-size: 70%"
                 }),
                   St.Align.MIDDLE
    )
    switchLayout.add(switchLabelLayout, { expand: true, x_fill:true, x_align: St.Align.START })
    switchLayout.add_actor(globalEnvSwitch._statusBin, { expand: true, x_fill:true, x_align: St.Align.END })
    globalEnvSwitch.addActor(switchLayout)
    
    //Wooo, has signals and slots^Hreceivers
    globalEnvSwitch.connect('toggled', _setPRIME)
    tbutton.menu.addMenuItem(globalEnvSwitch)
    tbutton.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem())
    
    switchLayout.width = gpuInfoLayout.layout.width
    dropArea._onCompleted(gpuInfoLayout.layout.width)

    let aboutPopup = new PopupMenu.PopupMenuItem("About")
    tbutton.menu.addMenuItem( aboutPopup )
    aboutPopup.connect('activate', _showAbout)
}

function init(){
    //clear, as per submission-time instructions
}

function enable() {
    _setup()
    Main.panel._addToPanelBox('', tbutton, 0, Main.panel._rightBox)
}

function disable() {
    Main.panel._rightBox.remove_child(tbutton)
}
